@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Cardiff University (2020)
+# Copyright (C) Cardiff University (2020-2022)
 #
 # This file is part of ciecplib.
 #
@@ -36,9 +36,11 @@ Valid starting       Expires              Service principal
 
 @pytest.mark.parametrize("side_effect, result", [
     (None, True),
-    (CalledProcessError(1, "klist"), False),
+    (CalledProcessError(1, "klist"), False),  # klist failed
+    (FileNotFoundError("no klist"), False),  # klist not found
 ])
 @mock.patch("subprocess.check_output")
+@mock.patch.dict("sys.modules", {"requests_ecp.auth": None})
 def test_has_credential(_check_output, side_effect, result):
     _check_output.side_effect = side_effect
     assert ciecplib_kerberos.has_credential("krb5ccname") is result
